"""Testing the pytest-nice plugin."""

import pytest


@pytest.fixture()
def sample_test(testdir):
    testdir.makepyfile(
        """
        from time import sleep
        def test_pass():
            sleep(0.15)
            assert 1 == 1

        def test_fail():
            sleep(0.01)
            assert 1 == 2
    """
    )
    return testdir


def test_without_slow(sample_test):
    result = sample_test.runpytest()
    result.stdout.fnmatch_lines(
        [
            "*.F*",
        ]
    )  # . for Pass, F for Fail
    assert result.ret == 1


def test_with_slow(sample_test):
    sample_test.runpytest("")
    result = sample_test.runpytest("--slow-last")
    result.stdout.fnmatch_lines(
        [
            "*F.*",
        ]
    )  # . for Pass, F for Fail
    assert result.ret == 1


def test_ff_preference(sample_test):
    sample_test.runpytest("")
    result = sample_test.runpytest("--slow-last", "--ff")
    result.stdout.fnmatch_lines(
        [
            "*F.*",
        ]
    )  # . for Pass, F for Fail
    assert result.ret == 1


def test_help_message(testdir):
    result = testdir.runpytest(
        "--help",
    )
    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "*Run slowest tests last*",
        ]
    )


def test_hello_ini_setting(testdir):
    testdir.makeini(
        """
        [pytest]
        addopts = --slow-last
    """
    )

    testdir.makepyfile(
        """
        from time import sleep
        def test_slow():
            sleep(0.15)

        def test_fast():
            sleep(0.01)
    """
    )

    testdir.runpytest("")
    result = testdir.runpytest("-v")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "*::test_fast PASSED*",
            "*::test_slow PASSED*",
        ]
    )

    # make sure that that we get a '0' exit code for the testsuite
    assert result.ret == 0


def test_slow_last_rounding(testdir):
    """Test the --slow-last-rounding option."""
    testdir.makepyfile(
        """
        from time import sleep
        def test_a():
            sleep(0.053)  # Just above threshold with default rounding

        def test_b():
            sleep(0.049)  # Just below threshold with default rounding

        def test_c():
            sleep(0.001)  # Definitely fast
    """
    )

    # First run to cache durations
    testdir.runpytest("")

    result = testdir.runpytest("--slow-last", "-v")
    result.stdout.fnmatch_lines(
        [
            "*::test_c PASSED*",
            "*::test_a PASSED*",
            "*::test_b PASSED*",
        ]
    )
    assert result.ret == 0

    # Run with custom rounding value
    result = testdir.runpytest("--slow-last", "--slow-last-rounding=3", "-v")
    result.stdout.fnmatch_lines(
        [
            "*::test_c PASSED*",
            "*::test_b PASSED*",
            "*::test_a PASSED*",
        ]
    )
    assert result.ret == 0



# -*- coding: utf-8 -*-

import datetime

import pytest


@pytest.fixture(scope="session")
def duration_cache(request):
    """We can't use `cache` fixture because it has function scope. However the `cache`
    fixture simply returns `request.config.cache`, which is available in any scope."""
    key = "duration/testdurations"
    d = request.config.cache.get(key, {})
    yield d
    request.config.cache.set(key, d)


@pytest.fixture(autouse=True)
def check_duration(request, duration_cache):
    """Stores the dureation of each test"""
    d = duration_cache
    nodeid = request.node.nodeid
    start_time = datetime.datetime.now()
    yield
    duration = (datetime.datetime.now() - start_time).total_seconds()
    d[nodeid] = duration


def by_duration(item):
    """Get the duration of a test from the cache"""
    d = item.config.cache.get("duration/testdurations", {}).get(item.nodeid, 0)
    rounding = item.config.getoption("--slow-last-rounding")
    if rounding >= 0:
        return round(d, rounding)
    else:
        return d


def pytest_addoption(parser):
    parser.addoption(
        "--slow-last",
        action="store_true",
        default=False,
        help="Run slowest tests last",
    )
    parser.addoption(
        "--slow-last-rounding",
        action="store",
        default=2,
        type=int,
        help="Number of decimal places to round test durations",
    )

def pytest_collection_modifyitems(items, config):
    """Sort tests by duration"""
    if config.getoption("--slow-last"):
        items.sort(key=by_duration, reverse=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import os

from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()


base_packages = ["pytest>=3.5.0"]

only_test_packages = [
    "black>=19.10b0",
    "flake8>=3.8.3",
]
test_packages = only_test_packages + base_packages

util_packages = ["pre-commit>=2.6.0", "ipykernel>=6.15.1", "twine"] + base_packages

docs_packages = [
    "mkdocs==1.2.3",
    "mkdocs-material==8.0.0",
    "mkdocstrings==0.18.0",
    "jinja2<3.1.0",
]

dev_packages = test_packages + util_packages + docs_packages

setup(
    name="pytest-slow-last",
    version="0.2.1",
    author="David Masip Bonet",
    author_email="david26694@gmail.com",
    maintainer="David Masip Bonet",
    maintainer_email="david26694@gmail.com",
    license="MIT",
    url="https://github.com/david26694/pytest-slow-last",
    description="Run tests in order of execution time (faster tests first)",
    long_description=read("README.rst"),
    py_modules=["pytest_slow_last"],
    python_requires=">=3.5",
    install_requires=base_packages,
    extras_require={
        "test": test_packages,
        "util": util_packages,
        "docs": docs_packages,
        "dev": dev_packages,
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={
        "pytest11": [
            "slow-last = pytest_slow_last",
        ],
    },
)
